c***********************************************************************
c
c  File:        test_patchmodule.f
c  Copyright:   (c) 2005-2007 Princeton University
c  Author(s):   Kevin T. Chu
c  Revision:    $Revision: 376 $
c  Modified:    $Date: 2007-08-06 19:10:55 -0400 (Mon, 06 Aug 2007) $
c  Description: F77 patch module routines for ElasticityModule test 
c               program
c
c***********************************************************************

c***********************************************************************
c  initdata() initializes the dislocation line data in the box given
c  the dislocation line direction, eta.
c
c  Arguments:
c    dislocation_line_* (out):  embedded dislocation line function
c    *_gb (in):                 ghostbox for dislocation line data
c    *_fb (in):                 fillbox for dislocation line data
c    eta (in):                  dislocation line direction
c    x_lower (in):              physical coordinates of lower corner 
c                               of box
c    dx (in):                   grid spacing
c
c***********************************************************************
      subroutine initdata(
     &  dislocation_line_x,
     &  dislocation_line_y,
     &  dislocation_line_z,
     &  ilo_gb, ihi_gb, jlo_gb, jhi_gb, klo_gb, khi_gb,
     &  ilo_fb, ihi_fb, jlo_fb, jhi_fb, klo_fb, khi_fb,
     &  eta,
     &  x_lower, 
     &  dx)
c***********************************************************************
      implicit none

c     _gb refers to ghost box
c     _fb refers to fill box
      integer ilo_gb, ihi_gb, jlo_gb, jhi_gb, klo_gb, khi_gb
      integer ilo_fb, ihi_fb, jlo_fb, jhi_fb, klo_fb, khi_fb
      double precision dislocation_line_x(ilo_gb:ihi_gb,
     &                                    jlo_gb:jhi_gb,
     &                                    klo_gb:khi_gb)
      double precision dislocation_line_y(ilo_gb:ihi_gb,
     &                                    jlo_gb:jhi_gb,
     &                                    klo_gb:khi_gb)
      double precision dislocation_line_z(ilo_gb:ihi_gb,
     &                                    jlo_gb:jhi_gb,
     &                                    klo_gb:khi_gb)
      double precision eta(1:3)
      double precision x_lower(1:3)
      double precision dx(1:3)
      double precision X_dot_eta
      double precision dist_X_to_disloc_line
      double precision x,y,z
      integer i,j,k
      double precision delta_width
      double precision delta
      double precision pi
      parameter (pi=3.14159265358979)


c     compute width of smoothed delta-function
      delta_width = (3*dx(1))
 
c     loop over grid {
      do k=klo_fb,khi_fb
        do j=jlo_fb,jhi_fb
          do i=ilo_fb,ihi_fb

            x = x_lower(1) + dx(1)*(i-ilo_fb+0.5)
            y = x_lower(2) + dx(2)*(j-jlo_fb+0.5)
            z = x_lower(3) + dx(3)*(k-klo_fb+0.5)

c           compute distance from point to dislocation line
            X_dot_eta = 
     &        x*eta(1) + y*eta(2) + z*eta(3)
  
            dist_X_to_disloc_line = 
     &        sqrt( (x*x+y*y+z*z) - X_dot_eta*X_dot_eta )

c           compute smoothed delta function of distance to 
c           dislocation line
            if (abs(dist_X_to_disloc_line) .gt. delta_width) then
              delta = 0
            else
              delta = 0.5/delta_width
     &              * ( 1+cos(pi*dist_X_to_disloc_line/delta_width) )
            endif

            dislocation_line_x(i,j,k) = delta*eta(1)
            dislocation_line_y(i,j,k) = delta*eta(2)
            dislocation_line_z(i,j,k) = delta*eta(3)

          enddo
        enddo
      enddo
c     } end loop over grid 

      return
      end
c***********************************************************************
