c***********************************************************************
c
c  File:        lsmdd_elastic_stress_routines.f
c  Copyright:   (c) 2005-2007 Princeton University
c  Author(s):   Kevin T. Chu
c  Revision:    $Revision: 376 $
c  Modified:    $Date: 2007-08-06 19:10:55 -0400 (Mon, 06 Aug 2007) $
c  Description: F77 utility subroutines for elastic stress calculations
c
c***********************************************************************

c***********************************************************************
c
c  lsmddAddStressComponent() adds the specified stress component to 
c  the same component of the total stress field.
c
c  Arguments:
c    total_stress (in/out):         component of total stress field
c    stress_contribution (in/out):  component of stress field to add
c                                   total stress
c    *_gb (in):                     index range for ghostbox
c    *_fb (in):                     index range for fillbox on which to 
c                                   add additional stress contribution
c
c***********************************************************************
      subroutine lsmddAddStressComponent(
     &  total_stress,
     &  ilo_total_stress_gb, ihi_total_stress_gb,
     &  jlo_total_stress_gb, jhi_total_stress_gb,
     &  klo_total_stress_gb, khi_total_stress_gb,
     &  stress_contribution,
     &  ilo_stress_contribution_gb, ihi_stress_contribution_gb,
     &  jlo_stress_contribution_gb, jhi_stress_contribution_gb,
     &  klo_stress_contribution_gb, khi_stress_contribution_gb,
     &  ilo_fb, ihi_fb,
     &  jlo_fb, jhi_fb,
     &  klo_fb, khi_fb)
c***********************************************************************
c { begin subroutine
      implicit none

c     _gb refers to ghostbox 
c     _fb refers to fillbox on which to compute unit tangent vector
      integer ilo_total_stress_gb, ihi_total_stress_gb
      integer jlo_total_stress_gb, jhi_total_stress_gb
      integer klo_total_stress_gb, khi_total_stress_gb
      integer ilo_stress_contribution_gb, ihi_stress_contribution_gb
      integer jlo_stress_contribution_gb, jhi_stress_contribution_gb
      integer klo_stress_contribution_gb, khi_stress_contribution_gb
      integer ilo_fb, ihi_fb
      integer jlo_fb, jhi_fb
      integer klo_fb, khi_fb
      double precision total_stress(
     &         ilo_total_stress_gb:ihi_total_stress_gb,
     &         jlo_total_stress_gb:jhi_total_stress_gb,
     &         klo_total_stress_gb:khi_total_stress_gb)
      double precision stress_contribution(
     &         ilo_stress_contribution_gb:ihi_stress_contribution_gb,
     &         jlo_stress_contribution_gb:jhi_stress_contribution_gb,
     &         klo_stress_contribution_gb:khi_stress_contribution_gb)
      integer i,j,k

c     loop over fillbox {
      do k=klo_fb,khi_fb
        do j=jlo_fb,jhi_fb
          do i=ilo_fb,ihi_fb

            total_stress(i,j,k) = total_stress(i,j,k) 
     &                          + stress_contribution(i,j,k)

          enddo
        enddo
      enddo
c     } end loop over grid 

      return
      end
c } end subroutine
c***********************************************************************
